@@ -1,11 +1,6 @@
-TEMPLATE_NAME=elf32
-SCRIPT_NAME=elf
-OUTPUT_FORMAT="elf32-powerpc"
-NO_REL_RELOCS=yes
-ARCH=powerpc
-MACHINE=
+. ${srcdir}/emulparams/elf32ppc.sh
+
 EMBEDDED=yes
-MAXPAGESIZE="CONSTANT (MAXPAGESIZE)"
 
 # The data below is taken from the windiss.dld linker script that comes with
 # the Diab linker.
